@@ -13,19 +13,15 @@ from pathlib import Path
 from typing import Any
 from typing import Optional
 from typing import Sequence
+from typing import Type
 
 import pandas as pd
 from anyio import to_thread
 from gwproto import Message
-from gwproto.enums import TelemetryName
-from gwproto.gs import GsPwr
 from gwproto.messages import EventBase
-from gwproto.messages import GtShStatusEvent
-from gwproto.messages import SnapshotSpaceheatEvent
-from gwproto.types import GtShStatus
-from gwproto.types import GtShStatus_Maker
-from gwproto.types import SnapshotSpaceheat
-from gwproto.types import SnapshotSpaceheat_Maker
+from gwproto.messages import ReportEvent
+from gwproto.named_types import SnapshotSpaceheat
+from pydantic import BaseModel
 from rich.console import RenderableType
 from rich.emoji import Emoji
 from rich.layout import Layout
@@ -111,6 +107,14 @@ class HoneywellThermostatOperatingState(Enum):
     fan_only = 6
 
 
+UNDISPLAYED_EVENTS = {
+    SnapshotSpaceheat.model_fields["TypeName"].default,
+    ReportEvent.model_fields["TypeName"].default,
+    "gridworks.event.gt.sh.status",
+    "gridworks.event.snapshot.spaceheat",
+}
+
+
 class TUI:
     settings: EventsSettings
     read_only: bool
@@ -125,7 +129,6 @@ class TUI:
     gwd_text: Text
     local_tz: timezone
     snaps: dict[str, SnapshotSpaceheat]
-    statuses: dict[str, GtShStatus]
     scadas_to_snap: list[str]
 
     def __init__(self, settings: EventsSettings, read_only: bool):
@@ -134,13 +137,13 @@ class TUI:
         if self.settings.paths.csv_path.exists():
             self.df = pd.read_csv(
                 self.settings.paths.csv_path,
-                index_col="TimeNS",
+                index_col="TimeCreatedMs",
                 parse_dates=True,
                 date_parser=functools.partial(pd.to_datetime, utc=True),
             )
         else:
             self.df = pd.DataFrame(
-                index=pd.DatetimeIndex([], name="TimeNS"),
+                index=pd.DatetimeIndex([], name="TimeCreatedMs"),
                 columns=["MessageId", "Src", "TypeName", "other_fields"],
             )
             self.df.to_csv(self.settings.paths.csv_path)
@@ -155,10 +158,11 @@ class TUI:
         self.event_table = self.make_event_table()
         self.load_snaps()
         self.select_scadas_for_snaps()
-        self.load_statuses()
         self.make_layout()
 
-    def _load_latest(self, suffix: str, member_name: str, maker: Any) -> None:
+    def _load_latest(
+        self, suffix: str, member_name: str, decoder: Type[BaseModel]
+    ) -> None:
         setattr(self, member_name, dict())
         member = getattr(self, member_name)
         latest_dir = getattr(self.settings.paths, f"{suffix}_dir")
@@ -167,13 +171,13 @@ class TUI:
             with path.open() as f:
                 latest_str = f.read()
             try:
-                made = maker.type_to_tuple(latest_str)
+                decoded = decoder.model_validate_json(latest_str)
             except Exception as e:
                 logger.exception("ERROR handling %s:\n%s\n", path, latest_str)
                 logger.exception(e)
                 # raise e
             else:
-                member[path.name[: -len(path_suffix)]] = made
+                member[path.name[: -len(path_suffix)]] = decoded
 
     def select_scadas_for_snaps(self):
         self.scadas_to_snap = []
@@ -192,11 +196,8 @@ class TUI:
             else:
                 self.scadas_to_snap.append("")
 
-    def load_statuses(self):
-        self._load_latest("status", "statuses", GtShStatus_Maker)
-
     def load_snaps(self):
-        self._load_latest("snap", "snaps", SnapshotSpaceheat_Maker)
+        self._load_latest("snap", "snaps", SnapshotSpaceheat)
 
     def extract_display_df(self) -> pd.DataFrame:
         if self.settings.scadas:
@@ -208,12 +209,13 @@ class TUI:
             filtered_df = self.df[self.df["Src"].isin(srcs_used)]
         else:
             filtered_df = self.df
+        filtered_df = filtered_df[~filtered_df["TypeName"].isin(UNDISPLAYED_EVENTS)]
         return filtered_df.tail(self.settings.tui.displayed_events)
 
     def reload_dfs(self):
         self.df = pd.read_csv(
             self.settings.paths.csv_path,
-            index_col="TimeNS",
+            index_col="TimeCreatedMs",
             parse_dates=True,
             date_parser=functools.partial(pd.to_datetime, utc=True),
         )
@@ -330,15 +332,18 @@ class TUI:
                 or row_df.index[0] >= self.display_df.index[0]
             ):
                 path_dbg |= 0x00000002
-                # Check if it is already present
-                if not (self.display_df["MessageId"] == message_id).any():
+                # Check if excluded by TypeName
+                if row_df["TypeName"][0] not in UNDISPLAYED_EVENTS:
                     path_dbg |= 0x00000004
-                    self.display_df = (
-                        pd.concat([self.display_df, row_df])
-                        .sort_index()
-                        .tail(self.settings.tui.displayed_events)
-                    )
-                    self.layout["events"].update(self.make_event_table())
+                    # Check if it is already present
+                    if not (self.display_df["MessageId"] == message_id).any():  # noqa
+                        path_dbg |= 0x00000008
+                        self.display_df = (
+                            pd.concat([self.display_df, row_df])
+                            .sort_index()
+                            .tail(self.settings.tui.displayed_events)
+                        )
+                        self.layout["events"].update(self.make_event_table())
         logger.debug(f"--update_display: 0x{path_dbg:08X}")
 
     def flush_live_history(self):
@@ -353,8 +358,8 @@ class TUI:
         path_dbg = 0
         if (
             not self.read_only
-            and not (self.live_history_df["MessageId"] == message_id).any()
-            and not (self.df["MessageId"] == message_id).any()
+            and not (self.live_history_df["MessageId"] == message_id).any()  # noqa
+            and not (self.df["MessageId"] == message_id).any()  # noqa
         ):
             path_dbg |= 0x00000001
             self.live_history_df = pd.concat(
@@ -368,17 +373,14 @@ class TUI:
     def handle_event(self, message_src: str, event: EventBase) -> None:
         logger.debug("++handle_event")
         if event.TypeName in ["gridworks.event.problem", "gridworks.event.shutdown"]:
-            logger.info(event.json(sort_keys=True, indent=2))
-        row_df = AnyEvent(**event.dict()).as_dataframe(
+            logger.info(event.model_dump_json(indent=2))
+        row_df = AnyEvent(**event.model_dump()).as_dataframe(
             columns=self.df.columns.values, interpolate_summary=True
         )
         self.update_display(message_src, event.MessageId, row_df)
         self.update_live_history(event.MessageId, row_df)
         logger.debug("--handle_event")
 
-    def handle_pwr(self, pwr: GsPwr):
-        pass
-
     def handle_snapshot(self, snap: SnapshotSpaceheat):
         logger.debug("++handle_snapshot")
         path_dbg = 0
@@ -394,10 +396,10 @@ class TUI:
                     stored_time = snap_dict.get("Snapshot", dict()).get(
                         "ReportTimeUnixMs", 0
                     )
-                    newer = snap.Snapshot.ReportTimeUnixMs > stored_time
+                    newer = snap.SnapshotTimeUnixMs > stored_time
             if newer:
                 path_dbg |= 0x00000004
-                snap_str = json.dumps(snap.dict(), sort_keys=True, indent=2)
+                snap_str = json.dumps(snap.model_dump(), sort_keys=True, indent=2)
                 with snapshot_path.open("w") as f:
                     f.write(snap_str)
                 self.snaps[snap.FromGNodeAlias] = snap
@@ -409,8 +411,8 @@ class TUI:
                         self.layout[f"snap{idx}"].update(
                             self.make_snapshot(snap.FromGNodeAlias)
                         )
-                logger.info(f"Snapshot from {snap.FromGNodeAlias}:")
-                logger.info(snap_str)
+                logger.debug(f"Snapshot from {snap.FromGNodeAlias}:")
+                logger.debug(snap_str)
         except Exception as e:
             path_dbg |= 0x00000020
             logger.exception(f"ERROR handling snapshot: {e}")
@@ -421,7 +423,7 @@ class TUI:
             return Panel("", border_style="blue")
         snap = self.snaps[name]
         report_time = (
-            pd.Timestamp(snap.Snapshot.ReportTimeUnixMs, unit="ms", tz="UTC")
+            pd.Timestamp(snap.SnapshotTimeUnixMs, unit="ms", tz="UTC")
             .tz_convert(self.local_tz)
             .strftime("%Y-%m-%d %X")
         )
@@ -436,83 +438,67 @@ class TUI:
             Column("Unit", header_style="orchid1", style="orchid1"),
             title=f"\nSnapshot at [green]{report_time}",
         )
-        for i in range(len(snap.Snapshot.AboutNodeAliasList)):
-            telemetry_name = snap.Snapshot.TelemetryNameList[i]
-            if (
-                telemetry_name == TelemetryName.WaterTempCTimes1000
-                or telemetry_name == TelemetryName.WaterTempCTimes1000.value
-                or telemetry_name == TelemetryName.AirTempCTimes1000
-                or telemetry_name == TelemetryName.AirTempCTimes1000.value
-            ):
-                centigrade = snap.Snapshot.ValueList[i] / 1000
-                if self.settings.tui.c_to_f:
-                    value_str = f"{(centigrade * 9/5) + 32:5.2f}"
-                    unit = "F"
-                else:
-                    value_str = f"{centigrade:5.2f}"
-                    unit = "C"
-            elif (
-                telemetry_name == TelemetryName.WaterTempFTimes1000
-                or telemetry_name == TelemetryName.WaterTempFTimes1000.value
-                or telemetry_name == TelemetryName.AirTempFTimes1000
-                or telemetry_name == TelemetryName.AirTempFTimes1000.value
-            ):
-                value_str = f"{snap.Snapshot.ValueList[i] / 1000:5.2f}"
-                unit = "F"
-            elif (
-                telemetry_name == TelemetryName.GallonsTimes100
-                or telemetry_name == TelemetryName.GallonsTimes100.value
-            ):
-                value_str = f"{snap.Snapshot.ValueList[i] / 100:5.2f}"
-                unit = "Gallons"
-            elif (
-                telemetry_name == TelemetryName.ThermostatState
-                or telemetry_name == TelemetryName.ThermostatState.value
-            ):
-                try:
-                    state_enum = HoneywellThermostatOperatingState(
-                        snap.Snapshot.ValueList[i]
-                    )
-                    enum_str = state_enum.name
-                except:  # noqa
-                    enum_str = "UNKNOWN"
-                value_str = f"{enum_str} / {snap.Snapshot.ValueList[i]}"
-                unit = "Heat State"
-            else:
-                value_str = f"{snap.Snapshot.ValueList[i]}"
-                unit = snap.Snapshot.TelemetryNameList[i].value
-            table.add_row(snap.Snapshot.AboutNodeAliasList[i], value_str, unit)
-
+        for i in range(len(snap.LatestReadingList)):
+            # requires access to channel list
+            # telemetry_name = snap.Snapshot.TelemetryNameList[i]
+            # if (
+            #     telemetry_name == TelemetryName.WaterTempCTimes1000
+            #     or telemetry_name == TelemetryName.WaterTempCTimes1000.value
+            #     or telemetry_name == TelemetryName.AirTempCTimes1000
+            #     or telemetry_name == TelemetryName.AirTempCTimes1000.value
+            # ):
+            #     centigrade = snap.Snapshot.ValueList[i] / 1000
+            #     if self.settings.tui.c_to_f:
+            #         value_str = f"{(centigrade * 9/5) + 32:5.2f}"
+            #         unit = "F"
+            #     else:
+            #         value_str = f"{centigrade:5.2f}"
+            #         unit = "C"
+            # elif (
+            #     telemetry_name == TelemetryName.WaterTempFTimes1000
+            #     or telemetry_name == TelemetryName.WaterTempFTimes1000.value
+            #     or telemetry_name == TelemetryName.AirTempFTimes1000
+            #     or telemetry_name == TelemetryName.AirTempFTimes1000.value
+            # ):
+            #     value_str = f"{snap.Snapshot.ValueList[i] / 1000:5.2f}"
+            #     unit = "F"
+            # elif (
+            #     telemetry_name == TelemetryName.GallonsTimes100
+            #     or telemetry_name == TelemetryName.GallonsTimes100.value
+            # ):
+            #     value_str = f"{snap.Snapshot.ValueList[i] / 100:5.2f}"
+            #     unit = "Gallons"
+            # elif (
+            #     telemetry_name == TelemetryName.ThermostatState
+            #     or telemetry_name == TelemetryName.ThermostatState.value
+            # ):
+            #     try:
+            #         state_enum = HoneywellThermostatOperatingState(
+            #             snap.Snapshot.ValueList[i]
+            #         )
+            #         enum_str = state_enum.name
+            #     except:  # noqa
+            #         enum_str = "UNKNOWN"
+            #     value_str = f"{enum_str} / {snap.Snapshot.ValueList[i]}"
+            #     unit = "Heat State"
+            # else:
+            #     value_str = f"{snap.Snapshot.ValueList[i]}"
+            #     unit = snap.Snapshot.TelemetryNameList[i].value
+            # table.add_row(snap.Snapshot.AboutNodeAliasList[i], value_str, unit)
+            table.add_row(
+                snap.LatestReadingList[i].ChannelName,
+                f"{snap.LatestReadingList[i].Value}",
+                "?",
+            )
         return Panel(table, title=f"[b]{snap.FromGNodeAlias}", border_style="blue")
 
-    def handle_status(self, status: GtShStatus):
-        try:
-            status_path = self.settings.paths.snap_path(status.FromGNodeAlias)
-            with status_path.open("w") as f:
-                f.write(json.dumps(status.dict(), sort_keys=True, indent=2))
-            self.statuses[status.FromGNodeAlias] = status
-        except Exception as e:
-            logger.exception(f"ERROR handling status: {e}")
-
     def handle_message(self, message: Message):
         logger.debug("++handle_message")
         path_dbg = 0
         match message.Payload:
-            case GsPwr():
-                path_dbg |= 0x00000001
-                self.handle_pwr(message.Payload)
             case SnapshotSpaceheat():
                 path_dbg |= 0x00000002
                 self.handle_snapshot(message.Payload)
-            case GtShStatus():
-                path_dbg |= 0x00000004
-                self.handle_status(message.Payload)
-            case GtShStatusEvent():
-                path_dbg |= 0x00000008
-                self.handle_status(message.Payload.status)
-            case SnapshotSpaceheatEvent():
-                path_dbg |= 0x00000010
-                self.handle_snapshot(message.Payload.snap)
             case EventBase():
                 path_dbg |= 0x00000020
                 self.handle_event(message.src(), message.Payload)
